@@ -9,7 +9,7 @@
 ## no critic (RequireUseStrict, RequireUseWarnings)
 package Riak::Client;
 {
-  $Riak::Client::VERSION = '1.92';
+  $Riak::Client::VERSION = '1.93';
 }
 ## use critic
 
@@ -94,21 +94,26 @@ has _handle => ( is => 'ro', lazy => 1, builder => 1 );
 sub _build__handle {
     my ($self) = @_;
     my ($host, $port) = ($self->host, $self->port);
-    my $handle;
 
     weaken $self;
 
+    my $handle;
+
+    my $on_error_handle = sub {
+        $handle->destroy; # explicitly destroy handle
+        my $command = $self->{_req_command} // "<unknown>";
+        my $bucket = $self->{_req_bucket} // "<unknown>";
+        my $key = $self->{_req_key} // "<unknown>";
+        croak("Error ($_[2]) on $host:$port, while performing: command '$command' on bucket '$bucket' and key '$key'");
+    };
+    weaken $on_error_handle;
+
+
     # TODO = timeouts
     $handle = AnyEvent::Handle->new (
       connect  => [$host, $port],
       no_delay => $self->no_delay(),
-      on_error => sub {
-         $handle->destroy; # explicitly destroy handle
-         my $command = $self->{_req_command} // "<unknown>";
-         my $bucket = $self->{_req_bucket} // "<unknown>";
-         my $key = $self->{_req_key} // "<unknown>";
-         croak("Error ($_[2]) on $host:$port, while performing: command '$command' on bucket '$bucket' and key '$key'");
-      },
+      on_error => $on_error_handle,
 #      rtimeout => $self->read_timeout,
 #      wtimeout => $self->write_timeout,
 #      on_prepare => sub { $self->connection_timeout },
@@ -116,6 +121,9 @@ sub _build__handle {
 #      on_timeout => sub { print STDERR " ---- PLOP \n";},
     );
 
+
+    $handle;
+
 }
 
 
@@ -126,12 +134,10 @@ sub _build__handle {
 has _current_request_ae_args => ( is => 'rw', default => sub { [] } );
 has _handle_reader_callback => ( is => 'ro', lazy => 1, builder => 1 );
 sub _build__handle_reader_callback {
-
     my ($self) = @_;
-
     weaken $self;
 
-    my $handle_reader_callback;
+    my $handle_reader_callback_weak;
 
     my $inner_handle_reader_callback = sub {
         my ( $response_code, $response_body ) = unpack( 'c a*', $_[1] );
@@ -162,7 +168,7 @@ sub _build__handle_reader_callback {
             ($ret, $more_to_come) = $handle_response->( $self, $response_body, $args );
         }
         # if we expect more to come, re-prepend the handler
-        $more_to_come and $_[0]->unshift_read( chunk => 4, $handle_reader_callback ),
+        $more_to_come and $_[0]->unshift_read( chunk => 4, $handle_reader_callback_weak),
           return;
     
         # ok, single or multiple response are over, remove the current request
@@ -183,15 +189,16 @@ sub _build__handle_reader_callback {
     
     };
 
-    $handle_reader_callback = sub {
+    my $handle_reader_callback = sub {
         # length arrived, decode
         my $len = unpack "N", $_[1];
         # now read the payload
         $_[0]->unshift_read( chunk => $len, $inner_handle_reader_callback);
     };
 
+    $handle_reader_callback_weak = $handle_reader_callback;
+    weaken $handle_reader_callback_weak;
     $handle_reader_callback;
-
 }
 
 has _socket => ( is => 'ro', lazy => 1, builder => 1 );
@@ -905,7 +912,7 @@ Riak::Client - Fast and lightweight Perl client for Riak
 
 =head1 VERSION
 
-version 1.92
+version 1.93
 
 =head1 SYNOPSIS
 
